@@ -8,6 +8,7 @@
 #include "lib/regcomp_dfa_regless.h"
 #include "lib/regex.h"
 #include "lib/regex_impl.h"
+#include "lib/regoff_trie.h"
 #include "src/debug/debug.h"
 #include "src/dfa/dfa.h"
 #include "src/msg/location.h"
@@ -37,8 +38,10 @@ int regcomp(regex_t *preg, const char *pattern, int cflags)
     globopts.FFlag = true;
     globopts.backward = cflags & REG_BACKWARD;
     globopts.stadfa = cflags & REG_STADFA;
-    Opt opts(globopts);
     Msg msg;
+    Opt opts(globopts, msg);
+    opts.set_subhistories((cflags & REG_SUBHIST) != 0);
+    opts.set_autotags((cflags & REG_AUTOTAGS) != 0);
     opts.set_posix_syntax(true);
     opts.set_posix_semantics((cflags & REG_LEFTMOST) == 0);
     const opt_t *opt = opts.snapshot();
@@ -54,26 +57,25 @@ int regcomp(regex_t *preg, const char *pattern, int cflags)
     RESpec re(arv, opt, msg, *preg->rmgr);
 
     find_fixed_tags(re);
-
     insert_default_tags(re);
 
-    nfa_t *nfa = new nfa_t(re);
+    size_t nfa_size, nfa_depth;
+    compute_size_and_depth(re.res, &nfa_size, &nfa_depth);
+    if (nfa_depth > MAX_NFA_DEPTH || nfa_size > MAX_NFA_STATES) return 1;
+
+    nfa_t *nfa = new nfa_t(re, nfa_size);
 
     nfa_t *nfa0 = NULL;
     if (cflags & REG_BACKWARD) {
         conopt_t globopts0;
         globopts0.FFlag = true;
-        Opt opts0(globopts0);
+        Opt opts0(globopts0, msg);
         const opt_t *opt0 = opts0.snapshot();
         RESpec re0(arv, opt0, msg, *preg->rmgr);
-        nfa0 = new nfa_t(re0);
+        nfa0 = new nfa_t(re0, nfa_size);
         delete opt0;
     }
 
-    DASSERT(nfa->rules.size() == 1);
-    preg->re_nsub = nfa->rules[0].ncap + 1;
-    preg->pmatch = new regmatch_t[preg->re_nsub];
-
     dfa_t *dfa = NULL;
     rldfa_t *rldfa = NULL;
 
@@ -101,20 +103,47 @@ int regcomp(regex_t *preg, const char *pattern, int cflags)
         dfa = new dfa_t(*nfa, Rule::NONE, Rule::NONE);
         if (cflags & REG_REGLESS) {
             DASSERT((cflags & REG_STADFA) == 0);
-            rldfa = new rldfa_t(*nfa, *dfa, opt);
+            rldfa = new rldfa_t(*nfa, *dfa, opt, cflags);
             opt = NULL; // transfer options ownership to RLDFA
         } else {
             determinization(*nfa, *dfa, opt, msg, "");
+            cutoff_dead_rules(*dfa, opt, "", msg);
+            insert_fallback_tags(opt, *dfa);
             compact_and_optimize_tags(opt, *dfa);
         }
 
-        preg->regs = new regoff_t[dfa->maxtagver + 1];
+        if (cflags & REG_TSTRING) {
+            // T-string does not need intermediate storage for tag values.
+        } else if (cflags & REG_SUBHIST) {
+            const size_t nlists = (cflags & REG_REGLESS)
+                ? dfa->tags.size() : static_cast<size_t>(dfa->maxtagver + 1);
+            preg->regtrie = new regoff_trie_t(nlists);
+        } else {
+            preg->regs = new regoff_t[dfa->maxtagver + 1];
+        }
     }
 
     preg->nfa = nfa;
     preg->dfa = dfa;
     preg->rldfa = rldfa;
 
+    DASSERT(nfa->rules.size() == 1);
+    preg->re_nsub = nfa->rules[0].ncap + 1;
+    preg->re_ntag = nfa->tags.size();
+
+    if (cflags & REG_TSTRING) {
+        // T-string is stored in RE (reallocated on each regtstring() call if
+        // needed), and the user gets an immutable view of it (const ref).
+        tstring_t &ts = preg->tstring;
+        ts.capacity = 256;
+        ts.string = new tchar_t[ts.capacity];
+        ts.length = 0;
+    } else if (cflags & REG_SUBHIST) {
+        // Allocated on every call to regparse() and returned to the user.
+    } else {
+        // Allocated by the user.
+    }
+
     delete opt;
     return 0;
 }
