@@ -6,6 +6,7 @@
 #include "lib/regcomp_dfa_regless.h"
 #include "lib/regex.h"
 #include "lib/regex_impl.h"
+#include "lib/regoff_trie.h"
 #include "src/dfa/dfa.h"
 #include "src/dfa/tcmd.h"
 #include "src/encoding/range_suffix.h"
@@ -29,7 +30,13 @@ void regfree(regex_t *preg)
     delete &preg->nfa->tags;
     delete preg->nfa;
 
-    delete[] preg->pmatch;
+    if (preg->flags & REG_TSTRING) {
+        delete[] preg->tstring.string;
+    } else if (preg->flags & REG_SUBHIST) {
+        // regfreesub() should be called by the user after regparse()
+    } else {
+        // allocated and freed by the user (pointer stored in regex_t)
+    }
 
     if (preg->flags & REG_NFA) {
         if ((preg->flags & REG_TRIE) && (preg->flags & REG_LEFTMOST)) {
@@ -44,7 +51,6 @@ void regfree(regex_t *preg)
             delete static_cast<psimctx_t*>(preg->simctx);
         }
     } else {
-        delete[] preg->regs;
         delete[] preg->char2class;
         delete[] preg->dfa->finvers;
 
@@ -62,8 +68,20 @@ void regfree(regex_t *preg)
             delete[] preg->rldfa->result;
             delete preg->rldfa;
         }
+        if (preg->flags & REG_TSTRING) {
+            // t-string construction does not use this
+        } else if (preg->flags & REG_SUBHIST) {
+            delete preg->regtrie;
+        } else {
+            delete[] preg->regs;
+        }
     }
 
     AST::flist.clear();
     RangeSuffix::freeList.clear();
 }
+
+void regfreesub(subhistory_t *history)
+{
+    free(history);
+}
