@@ -1,6 +1,7 @@
 #ifndef _RE2C_CODEGEN_CODE_
 #define _RE2C_CODEGEN_CODE_
 
+#include <assert.h>
 #include <stddef.h>
 #include "src/util/c99_stdint.h"
 #include <string.h>
@@ -9,14 +10,16 @@
 #include <string>
 #include <sstream>
 #include <vector>
+#include <map>
 
+#include "src/constants.h"
 #include "src/dfa/tcmd.h"
 #include "src/debug/debug.h"
 #include "src/msg/location.h"
 #include "src/util/forbid_copy.h"
 #include "src/util/slab_allocator.h"
-#include "src/util/uniq_vector.h"
 #include "src/util/smart_ptr.h"
+#include "src/util/string_utils.h"
 
 
 namespace re2c {
@@ -28,7 +31,8 @@ struct opt_t;
 struct Code;
 struct CodeGo;
 struct CodeGoIf;
-class Output;
+struct Output;
+struct OutputBlock;
 struct State;
 struct DFA;
 typedef std::vector<smart_ptr<DFA> > dfas_t;
@@ -38,6 +42,8 @@ template <typename value_t> class uniq_vector_t;
 typedef slab_allocator_t<1024 * 1024, 8> code_alc_t;
 
 struct Label {
+    static const uint32_t NONE = ~0u;
+
     uint32_t index;
     bool     used;
 };
@@ -64,10 +70,15 @@ public:
     Scratchbuf& u64(uint64_t u) { os << u; return *this; }
     Scratchbuf& str(const std::string &s) { os << s; return *this; }
     Scratchbuf& cstr(const char *s) { os << s; return *this; }
-    Scratchbuf& label(const Label &l) { os << l.index; return *this; }
     Scratchbuf& u32_width(uint32_t u, int width);
     Scratchbuf& yybm_char(uint32_t u, const opt_t *opts, int width);
     Scratchbuf& exact_uint(size_t width);
+
+    Scratchbuf& label(const Label &l, bool check_used = true) {
+        assert((l.used || !check_used) && l.index != Label::NONE);
+        os << l.index;
+        return *this;
+    }
 };
 
 template<typename T>
@@ -108,10 +119,15 @@ struct Span {
     tcid_t    tags;
 };
 
+struct CodeRanges {
+    VarType   type;
+    uint32_t  size;
+    int64_t  *elems;
+};
+
 struct CodeGoCase {
-    uint32_t  nranges;
-    uint32_t *ranges;
-    CodeJump  jump;
+    const CodeRanges *ranges;
+    CodeJump          jump;
 };
 
 struct CodeGoSw {
@@ -227,7 +243,7 @@ struct CodeCase {
     } kind;
 
     union {
-        const CodeGoCase *gocase;
+        const CodeRanges *ranges;
         int32_t           number;
         const char       *string;
     };
@@ -254,14 +270,20 @@ struct CodeBlock {
 };
 
 struct CodeVar {
-    const char *type;
+    VarType type;
     const char *name;
     const char *init;
 };
 
-struct CodeTags {
-    const char *fmt;
-    const char *sep;
+struct BlockNameList {
+    const char    *name;
+    BlockNameList *next;
+};
+
+struct CodeFmt {
+    BlockNameList *block_names;
+    const char    *format;
+    const char    *separator;
 };
 
 struct CodeRaw {
@@ -295,56 +317,23 @@ struct CodeLabel {
 };
 
 struct Code {
-    enum Kind {
-        EMPTY,
-        IF_THEN_ELSE,
-        SWITCH,
-        BLOCK,
-        FUNC,
-        SKIP,
-        PEEK,
-        BACKUP,
-        PEEK_SKIP,
-        SKIP_PEEK,
-        SKIP_BACKUP,
-        BACKUP_SKIP,
-        BACKUP_PEEK,
-        BACKUP_PEEK_SKIP,
-        SKIP_BACKUP_PEEK,
-        LINE_INFO_INPUT,
-        LINE_INFO_OUTPUT,
-        COND_ENUM,
-        COND_GOTO,
-        COND_TABLE,
-        STATE_GOTO,
-        STAGS,
-        MTAGS,
-        YYMAXFILL,
-        YYMAXNMATCH,
-        YYCH,
-        YYACCEPT,
-        VAR,
-        STMT,
-        TEXT,
-        TEXT_RAW,
-        RAW,
-        LABEL
-    } kind;
-
     union {
-        const char *text;
-        CodeIfTE    ifte;
-        CodeSwitch  swch;
-        CodeBlock   block;
-        CodeFunc    func;
-        CodeRaw     raw;
-        CodeVar     var;
-        CodeTags    tags;
-        CodeLabel   label;
-        loc_t       loc;
+        const char    *text;
+        BlockNameList *block_names;
+        CodeIfTE       ifte;
+        CodeSwitch     swch;
+        CodeBlock      block;
+        CodeFunc       func;
+        CodeRaw        raw;
+        CodeVar        var;
+        CodeFmt        fmt;
+        CodeLabel      label;
+        CodeList      *loop;
+        loc_t          loc;
     };
 
-    Code *next;
+    CodeKind  kind;
+    Code     *next;
 };
 
 template<typename T>
@@ -384,7 +373,7 @@ inline void append(code_list_t<T> *list1, code_list_t<T> *list2)
     }
 }
 
-inline Code *new_code(code_alc_t &alc, Code::Kind kind)
+inline Code *new_code(code_alc_t &alc, CodeKind kind)
 {
     Code *x = alc.alloct<Code>(1);
     x->kind = kind;
@@ -394,21 +383,28 @@ inline Code *new_code(code_alc_t &alc, Code::Kind kind)
 
 inline Code *code_stmt(code_alc_t &alc, const char *text)
 {
-    Code *x = new_code(alc, Code::STMT);
+    Code *x = new_code(alc, CODE_STMT);
     x->text = text;
     return x;
 }
 
+inline Code *code_loop(code_alc_t &alc, CodeList *loop)
+{
+    Code *x = new_code(alc, CODE_LOOP);
+    x->loop = loop;
+    return x;
+}
+
 inline Code *code_text(code_alc_t &alc, const char *text)
 {
-    Code *x = new_code(alc, Code::TEXT);
+    Code *x = new_code(alc, CODE_TEXT);
     x->text = text;
     return x;
 }
 
 inline Code *code_raw(code_alc_t &alc, const char *data, size_t size)
 {
-    Code *x = new_code(alc, Code::RAW);
+    Code *x = new_code(alc, CODE_RAW);
     char *copy = alc.alloct<char>(size);
     memcpy(copy, data, size);
     x->raw.size = size;
@@ -418,11 +414,16 @@ inline Code *code_raw(code_alc_t &alc, const char *data, size_t size)
 
 inline Code *code_textraw(code_alc_t &alc, const char *text)
 {
-    Code *x = new_code(alc, Code::TEXT_RAW);
+    Code *x = new_code(alc, CODE_TEXT_RAW);
     x->text = text;
     return x;
 }
 
+inline Code *code_abort(code_alc_t &alc)
+{
+    return new_code(alc, CODE_ABORT);
+}
+
 inline Code *code_newline(code_alc_t &alc)
 {
     return code_textraw(alc, "");
@@ -430,7 +431,7 @@ inline Code *code_newline(code_alc_t &alc)
 
 inline Code *code_nlabel(code_alc_t &alc, Label *label)
 {
-    Code *x = new_code(alc, Code::LABEL);
+    Code *x = new_code(alc, CODE_LABEL);
     x->label.kind = CodeLabel::NLABEL;
     x->label.nlabel = label;
     return x;
@@ -438,100 +439,84 @@ inline Code *code_nlabel(code_alc_t &alc, Label *label)
 
 inline Code *code_slabel(code_alc_t &alc, const char *label)
 {
-    Code *x = new_code(alc, Code::LABEL);
+    Code *x = new_code(alc, CODE_LABEL);
     x->label.kind = CodeLabel::SLABEL;
     x->label.slabel = label;
     return x;
 }
 
-inline Code *code_tags(code_alc_t &alc, const std::string &fmt, const std::string &sep,
-    bool mtags)
+inline Code *code_fmt(code_alc_t &alc, CodeKind kind, BlockNameList *blocks,
+    const char *format, const char *separator)
 {
-    Code *x = new_code(alc, mtags ? Code::MTAGS : Code::STAGS);
-
-    const size_t fmt_size = fmt.length() + 1;
-    char *fmt_copy = alc.alloct<char>(fmt_size);
-    memcpy(fmt_copy, fmt.c_str(), fmt_size);
-    x->tags.fmt = fmt_copy;
-
-    const size_t sep_size = sep.length() + 1;
-    char *sep_copy = alc.alloct<char>(sep_size);
-    memcpy(sep_copy, sep.c_str(), sep_size);
-    x->tags.sep = sep_copy;
-
+    Code *x = new_code(alc, kind);
+    x->fmt.block_names = blocks;
+    x->fmt.format      = format;
+    x->fmt.separator   = separator;
     return x;
 }
 
-inline Code *code_yymaxfill(code_alc_t &alc)
-{
-    return new_code(alc, Code::YYMAXFILL);
-}
-
-inline Code *code_yymaxnmatch(code_alc_t &alc)
-{
-    return new_code(alc, Code::YYMAXNMATCH);
-}
-
-inline Code *code_cond_enum(code_alc_t &alc)
-{
-    return new_code(alc, Code::COND_ENUM);
-}
-
 inline Code *code_cond_table(code_alc_t &alc)
 {
-    return new_code(alc, Code::COND_TABLE);
+    return new_code(alc, CODE_COND_TABLE);
 }
 
 inline Code *code_cond_goto(code_alc_t &alc)
 {
-    return new_code(alc, Code::COND_GOTO);
+    return new_code(alc, CODE_COND_GOTO);
 }
 
-inline Code *code_state_goto(code_alc_t &alc)
+inline Code *code_state_goto(code_alc_t &alc, BlockNameList *blocks)
 {
-    return new_code(alc, Code::STATE_GOTO);
+    Code *x = new_code(alc, CODE_STATE_GOTO);
+    x->block_names = blocks;
+    return x;
 }
 
 inline Code *code_line_info_input(code_alc_t &alc, const loc_t &loc)
 {
-    Code *x = new_code(alc, Code::LINE_INFO_INPUT);
+    Code *x = new_code(alc, CODE_LINE_INFO_INPUT);
     x->loc = loc;
     return x;
 }
 
 inline Code *code_yych_decl(code_alc_t &alc)
 {
-    return new_code(alc, Code::YYCH);
+    return new_code(alc, CODE_YYCH);
 }
 
 inline Code *code_yyaccept_def(code_alc_t &alc)
 {
-    return new_code(alc, Code::YYACCEPT);
+    return new_code(alc, CODE_YYACCEPT);
+}
+
+inline Code *code_yystate_def(code_alc_t &alc)
+{
+    return new_code(alc, CODE_YYSTATE);
 }
 
 inline Code *code_line_info_output(code_alc_t &alc)
 {
-    return new_code(alc, Code::LINE_INFO_OUTPUT);
+    return new_code(alc, CODE_LINE_INFO_OUTPUT);
 }
 
 inline Code *code_skip(code_alc_t &alc)
 {
-    return new_code(alc, Code::SKIP);
+    return new_code(alc, CODE_SKIP);
 }
 
 inline Code *code_peek(code_alc_t &alc)
 {
-    return new_code(alc, Code::PEEK);
+    return new_code(alc, CODE_PEEK);
 }
 
 inline Code *code_backup(code_alc_t &alc)
 {
-    return new_code(alc, Code::BACKUP);
+    return new_code(alc, CODE_BACKUP);
 }
 
 inline Code *code_block(code_alc_t &alc, CodeList *stmts, CodeBlock::Fmt fmt)
 {
-    Code *x = new_code(alc, Code::BLOCK);
+    Code *x = new_code(alc, CODE_BLOCK);
     x->block.stmts = stmts;
     x->block.fmt   = fmt;
     return x;
@@ -540,7 +525,7 @@ inline Code *code_block(code_alc_t &alc, CodeList *stmts, CodeBlock::Fmt fmt)
 inline Code *code_if_then_else(code_alc_t &alc, const char *if_cond, CodeList *if_code,
     CodeList *else_code, bool oneline = true)
 {
-    Code *x = new_code(alc, Code::IF_THEN_ELSE);
+    Code *x = new_code(alc, CODE_IF_THEN_ELSE);
     x->ifte.if_cond   = if_cond;
     x->ifte.else_cond = NULL;
     x->ifte.if_code   = if_code;
@@ -585,11 +570,19 @@ inline CodeCase *code_case_string(code_alc_t &alc, CodeList *body, const char *s
     return x;
 }
 
-inline CodeCase *code_case_ranges(code_alc_t &alc, CodeList *body,
-    const CodeGoCase *gocase)
-{
+inline CodeCase *code_case_ranges(
+        code_alc_t &alc, CodeList *body, const CodeRanges *ranges) {
     CodeCase *x = code_case(alc, body, CodeCase::RANGES);
-    x->gocase = gocase;
+    x->ranges = ranges;
+    return x;
+}
+
+inline CodeRanges *code_ranges(
+        code_alc_t &alc, VarType type, int64_t *start, int64_t *end) {
+    CodeRanges *x = alc.alloct<CodeRanges>(1);
+    x->size = static_cast<uint32_t>(end - start) / 2;
+    x->type = type;
+    x->elems = start;
     return x;
 }
 
@@ -614,7 +607,7 @@ inline CodeArgs *code_args(code_alc_t &alc)
 inline Code *code_func(code_alc_t &alc, const char *name, CodeArgs *args,
     const char *semi)
 {
-    Code *x = new_code(alc, Code::FUNC);
+    Code *x = new_code(alc, CODE_FUNC);
     x->func.args = args;
     x->func.name = name;
     x->func.semi = semi;
@@ -634,7 +627,7 @@ inline Code *code_fcall(code_alc_t &alc, const char *name, CodeArgs *args,
 
 inline Code *code_switch(code_alc_t &alc, const char *expr, CodeCases *cases)
 {
-    Code *x = new_code(alc, Code::SWITCH);
+    Code *x = new_code(alc, CODE_SWITCH);
     x->swch.expr   = expr;
     x->swch.cases  = cases;
     return x;
@@ -665,23 +658,31 @@ inline CodeBitmap *code_bitmap(code_alc_t &alc, uint32_t nchars)
     return x;
 }
 
-struct CodegenContext {
+typedef std::vector<OutputBlock *> blocks_t;
+typedef blocks_t::const_iterator blocks_citer_t;
+typedef std::set<std::string> tagnames_t;
+
+struct CodegenCtxGlobal {
     code_alc_t &allocator;
     Scratchbuf &scratchbuf;
-    const opt_t *globopts;
-    const opt_t *opts;
     Msg &msg;
-    const loc_t &loc;
-    const uniq_vector_t<std::string> &allcondnames;
-    const std::set<std::string> &allstags;
-    const std::set<std::string> &allmtags;
-    const std::vector<std::string> &condnames;
-    const size_t maxfill;
-    const size_t maxnmatch;
-    const bool used_yyaccept;
+    const opt_t *opts;
+    const blocks_t &cblocks;
+    const blocks_t &hblocks;
+    const blocks_t *pblocks;
+    mutable blocks_t tmpblocks; // temporary buffer for block lists in directives
     const bool warn_cond_ord;
-    const uint32_t fill_index;
-    const std::vector<CodeList*> &fill_goto;
+};
+
+struct CodegenCtxPass1 {
+    const CodegenCtxGlobal *global; // global part
+    const OutputBlock *block;       // block-local part
+};
+
+struct CodegenCtxPass2 {
+    code_alc_t &allocator;
+    Scratchbuf &scratchbuf;
+    const opt_t *opts;
 };
 
 struct RenderContext {
@@ -698,51 +699,58 @@ struct OutputFragment {
     uint32_t  indent;
 };
 
+struct StartCond {
+    std::string name;
+    uint32_t number;
+};
+typedef std::vector<StartCond> StartConds;
+
+typedef std::map<uint32_t, CodeList*> storable_states_t;
+
 struct OutputBlock {
+    InputBlockKind kind;
+    std::string name;
     const loc_t loc;
     std::vector<OutputFragment> fragments;
     bool used_yyaccept;
     bool have_user_code;
-    bool is_reuse_block;
-    std::vector<std::string> types;
-    std::set<std::string> stags;
-    std::set<std::string> mtags;
+    StartConds conds;
+    tagnames_t stags;
+    tagnames_t mtags;
     const opt_t *opts;
 
-    // used in state dispatch
-    uint32_t fill_index;              // upper bound of YYFILL state index
-    std::vector<CodeList*> fill_goto; // transitions to YYFILL states
+    size_t max_fill;   // YYMAXFILL
+    size_t max_nmatch; // YYMAXNMATCH
 
-    OutputBlock(const loc_t &loc, bool reuse);
+    // Used in the state switch (with `-f --storable-state` option).
+    Label *start_label; // label of the DFA start state
+    uint32_t fill_index; // next free index in YYFILL state enumeration
+    uint32_t fill_state; // index of the most recent YYFILL state
+    storable_states_t fill_goto; // transitions to YYFILL states
+
+    OutputBlock(InputBlockKind kind, const std::string &name, const loc_t &loc);
     ~OutputBlock();
     FORBID_COPY(OutputBlock);
 };
 
-typedef std::vector<OutputBlock *> blocks_t;
-typedef blocks_t::const_iterator blocks_citer_t;
-
-class Output {
+struct Output {
     blocks_t cblocks; /* .c file */
     blocks_t hblocks; /* .h file */
     blocks_t *pblocks; /* selector */
-
-public:
     uint32_t label_counter;
     bool state_goto;
-    bool cond_enum_in_hdr;
+    bool cond_enum_autogen; // true unless an explicit `types:re2c` directive is used
     bool cond_goto;
     bool warn_condition_order;
     bool need_header;
+    bool done_mtag_defs;
     Msg &msg;
     std::set<std::string> skeletons;
-    size_t max_fill;
-    size_t max_nmatch;
     code_alc_t allocator;
     Scratchbuf scratchbuf;
 
-    // used in state dispatch (accumulated for all non-reuse blocks)
-    uint32_t total_fill_index;              // upper bound of YYFILL state index
-    std::vector<CodeList*> total_fill_goto; // transitions to YYFILL states
+    // YYFILL state index accumulated for all non-reuse blocks
+    uint32_t total_fill_index;
 
     // "final" options accumulated for all non-reuse blocks
     const opt_t *total_opts;
@@ -750,34 +758,31 @@ public:
     explicit Output(Msg &msg);
     ~Output();
     OutputBlock &block();
-    size_t blockid() const;
     bool open ();
-    void new_block(Opt &opts, const loc_t &loc, bool reuse);
+    void new_block(Opt &opts, InputBlockKind kind, const std::string &name,
+        const loc_t &loc);
     void gather_info_from_block();
     void header_mode(bool on);
     bool in_header() const;
-    void wraw (const char *s, const char *e);
+    void wraw(const char *s, const char *e, bool newline = false);
     void wversion_time ();
     void wdelay_stmt(uint32_t ind, Code *code);
     bool emit();
-    bool emit_blocks(const std::string &fname, blocks_t &blocks,
-        const uniq_vector_t<std::string> &global_types,
-        const std::set<std::string> &global_stags,
-        const std::set<std::string> &global_mtags);
+    bool emit_blocks(const std::string &fname, const CodegenCtxGlobal &globalctx);
     FORBID_COPY (Output);
 };
 
 void init_go(CodeGo *go);
-void code_go(code_alc_t &alc, CodeGo *go, const State *from, const opt_t *opts,
-    CodeBitmap *bitmap);
+void code_go(code_alc_t &alc, const DFA &dfa, const opt_t *opts, State *from);
+State *fallback_state_with_eof_rule(
+        const DFA &dfa, const opt_t *opts, const State *state, tcid_t *ptags);
 CodeBmState *find_bitmap(const CodeBitmap *bitmap, const CodeGo *go, const State *s);
 void insert_bitmap(code_alc_t &alc, CodeBitmap *bitmap, const CodeGo *go, const State *s);
 CodeList *gen_bitmap(Output &output, const CodeBitmap *bitmap);
 
 void gen_go(Output &output, const DFA &dfa, const CodeGo *go, const State *from,
     CodeList *stmts);
-void gen_tags(Scratchbuf &o, const opt_t *opts, Code *code,
-    const std::set<std::string> &tags);
+void gen_tags(Scratchbuf &buf, const opt_t *opts, Code *code, const tagnames_t &tags);
 void emit_action(Output &output, const DFA &dfa, const State *s, CodeList *stmts);
 void gen_settags(Output &output, CodeList *tag_actions, const DFA &dfa, tcid_t tcid,
     bool delayed);
@@ -785,16 +790,25 @@ void gen_goto(Output &output, const DFA &dfa, CodeList *stmts, const State *from
     const CodeJump &jump);
 const char *gen_lessthan(Scratchbuf &o, const opt_t *opts, size_t n);
 void gen_code(Output &output, dfas_t &dfas);
+void gen_dfa_as_blocks_with_labels(Output &output, const DFA &dfa, CodeList *stmts);
+void gen_dfa_as_switch_cases(Output &output, DFA &dfa, CodeCases *cases);
+void wrap_dfas_in_loop_switch(Output &output, CodeList *stmts, CodeCases *cases);
 
-void expand(CodegenContext &ctx, Code *code);
-void combine(CodegenContext &ctx, Code *code);
+void expand_pass_1(CodegenCtxPass1 &ctx, Code *code);
+void expand_pass_2(CodegenCtxPass2 &ctx, Code *code);
+void remove_empty(CodegenCtxPass2 &ctx, Code *code);
+void combine(CodegenCtxPass2 &ctx, Code *code);
 void render(RenderContext &rctx, const Code *code);
 
 bool consume(const State *s);
 void expand_fintags(const Tag &tag, std::vector<std::string> &fintags);
-std::string vartag_name(tagver_t ver, const std::string &prefix);
-std::string vartag_expr(tagver_t ver, const opt_t *opts);
+std::string vartag_name(tagver_t ver, const std::string &prefix,
+    const std::set<tagver_t> &mtagvers);
+std::string vartag_expr(tagver_t ver, const opt_t *opts,
+    const std::set<tagver_t> &mtagvers);
 void output_version_time(std::ostream &os, const opt_t *opts);
+void gen_peek_expr(std::ostream &os, const opt_t *opts);
+void gen_yydebug(Output &output, const Label *label, CodeList *stmts);
 
 } // namespace re2c
 
