@@ -9,7 +9,6 @@
 #include <valarray>
 #include <vector>
 
-#include "src/adfa/action.h"
 #include "src/adfa/adfa.h"
 #include "src/codegen/code.h"
 #include "src/codegen/helpers.h"
@@ -31,19 +30,31 @@ static void emit_accept(Output &output, CodeList *stmts, const DFA &dfa,
 static void emit_rule(Output &output, CodeList *stmts, const DFA &dfa, size_t rule_idx);
 static void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa,
     const Rule &rule);
-static bool endstate(const State *s);
 
 static const char *gen_fill_label(Output &output, uint32_t index)
 {
     const opt_t *opts = output.block().opts;
     Scratchbuf &o = output.scratchbuf;
     DASSERT(o.empty());
+    return o.str(opts->yyfilllabel).u32(index).flush();
+}
 
-    o.str(opts->yyfilllabel);
-    if (output.block().is_reuse_block) o.u64(output.blockid()).cstr("_");
-    o.u32(index);
+static bool endstate(const State *s) {
+    // An 'end' state is a state which has no outgoing transitions on symbols.
+    // Usually 'end' states are final states (not all final states are 'end' states),
+    // but sometimes it be initial non-accepting state, e.g. in case of rule '[]'.
+    DASSERT(s->go.nspans > 0);
+    const Action::type_t &a = s->go.span[0].to->action.type;
+    return s->go.nspans == 1 && (a == Action::RULE || a == Action::ACCEPT);
+}
 
-    return o.flush();
+static void gen_peek(code_alc_t &alc, const State *s, CodeList *stmts) {
+    // Do not generate YYPEEK statement in case `yych` is overwritten before it is used.
+    // This may happen if there is a single transition which does not require matching on
+    // `yych` (one exception is a transition to a move state, which doesn't have its own
+    // YYPEEK and relies on the previous value of `yych`).
+    bool omit_peek = s->go.nspans == 1 && s->go.span[0].to->action.type != Action::MOVE;
+    if (!omit_peek) append(stmts, code_peek(alc));
 }
 
 void emit_action(Output &output, const DFA &dfa, const State *s, CodeList *stmts)
@@ -59,9 +70,7 @@ void emit_action(Output &output, const DFA &dfa, const State *s, CodeList *stmts
             append(stmts, code_skip(alc));
         }
         gen_fill_and_label(output, stmts, dfa, s);
-        if (!endstate(s)) {
-            append(stmts, code_peek(alc));
-        }
+        gen_peek(alc, s, stmts);
         break;
     case Action::INITIAL: {
         const size_t save = s->action.info.save;
@@ -76,18 +85,12 @@ void emit_action(Output &output, const DFA &dfa, const State *s, CodeList *stmts
             append(stmts, code_skip(alc));
         }
         append(stmts, code_nlabel(alc, dfa.initial_label));
-        if (opts->dFlag) {
-            text = o.str(opts->yydebug).cstr("(").label(*dfa.initial_label)
-                .cstr(", *").str(opts->yycursor).cstr(")").flush();
-            append(stmts, code_stmt(alc, text));
-        }
+        gen_yydebug(output, dfa.initial_label, stmts);
         gen_fill_and_label(output, stmts, dfa, s);
         if (backup) {
             append(stmts, code_backup(alc));
         }
-        if (!endstate(s)) {
-            append(stmts, code_peek(alc));
-        }
+        gen_peek(alc, s, stmts);
         break;
     }
     case Action::SAVE:
@@ -100,7 +103,7 @@ void emit_action(Output &output, const DFA &dfa, const State *s, CodeList *stmts
         }
         append(stmts, code_backup(alc));
         gen_fill_and_label(output, stmts, dfa, s);
-        append(stmts, code_peek(alc));
+        gen_peek(alc, s, stmts);
         break;
     case Action::MOVE:
         break;
@@ -194,7 +197,6 @@ void emit_accept(Output &output, CodeList *stmts, const DFA &dfa, const accept_t
         append(block, code_text(alc, text));
         for (uint32_t i = 0; i < nacc; ++i) {
             Label *l = acc[i].first->label;
-            l->used = true;
             text = o.cstr("&&").str(opts->labelPrefix).label(*l).cstr(",").flush();
             append(table, code_text(alc, text));
         }
@@ -232,54 +234,7 @@ void emit_accept(Output &output, CodeList *stmts, const DFA &dfa, const accept_t
     append(stmts, code_switch(alc, text, cases));
 }
 
-void emit_rule(Output &output, CodeList *stmts, const DFA &dfa, size_t rule_idx)
-{
-    const opt_t *opts = output.block().opts;
-    const Rule &rule = dfa.rules[rule_idx];
-    const SemAct *semact = rule.semact;
-    const std::string &cond = semact->cond;
-    code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
-    std::string s;
-    const char *text;
-
-    gen_fintags(output, stmts, dfa, rule);
-
-    if (opts->target == TARGET_SKELETON) {
-        emit_skeleton_action(output, stmts, dfa, rule_idx);
-    }
-    else {
-        if (!cond.empty() && dfa.cond != cond) {
-            o.str(opts->cond_set);
-            argsubst(o.stream(), opts->cond_set_arg, "cond", true,
-                opts->condEnumPrefix + cond);
-            if (opts->cond_set_naked) {
-                append(stmts, code_text(alc, o.flush()));
-            } else {
-                o.cstr("(").str(opts->condEnumPrefix).str(cond).cstr(")");
-                append(stmts, code_stmt(alc, o.flush()));
-            }
-        }
-        if (!semact->autogen) {
-            if (!dfa.setup.empty()) {
-                text = o.str(dfa.setup).flush();
-                append(stmts, code_text(alc, text));
-            }
-            append(stmts, code_line_info_input(alc, semact->loc));
-            text = o.str(semact->text).flush();
-            append(stmts, code_text(alc, text));
-            append(stmts, code_line_info_output(alc));
-        }
-        else if (!cond.empty()) {
-            o.str(opts->condGoto);
-            argsubst(o.stream(), opts->condGotoParam, "cond", true,
-                opts->condPrefix + cond);
-            append(stmts, code_text(alc, o.flush()));
-        }
-    }
-}
-
-static void gen_setstate(Output &output, CodeList *stmts, uint32_t fillidx)
+static void gen_setstate(Output &output, CodeList *stmts, const char *fillidx)
 {
     const opt_t *opts = output.block().opts;
     code_alc_t &alc = output.allocator;
@@ -290,33 +245,112 @@ static void gen_setstate(Output &output, CodeList *stmts, uint32_t fillidx)
     if (opts->state_set_naked) {
         append(stmts, code_text(alc, o.flush()));
     } else {
-        o.cstr("(").u32(fillidx).cstr(")");
+        o.cstr("(").cstr(fillidx).cstr(")");
         append(stmts, code_stmt(alc, o.flush()));
     }
 }
 
+static void gen_setcondition(Output &output, CodeList *stmts, const char *cond)
+{
+    const opt_t *opts = output.block().opts;
+    code_alc_t &alc = output.allocator;
+    Scratchbuf &o = output.scratchbuf;
+
+    o.str(opts->cond_set);
+    argsubst(o.stream(), opts->cond_set_arg, "cond", true, cond);
+    if (opts->cond_set_naked) {
+        append(stmts, code_text(alc, o.flush()));
+    } else {
+        o.cstr("(").cstr(cond).cstr(")");
+        append(stmts, code_stmt(alc, o.flush()));
+    }
+}
+
+static void gen_continue_yyloop(Output &output, CodeList *stmts, const char *next) {
+    const opt_t *opts = output.block().opts;
+    code_alc_t &alc = output.allocator;
+    Scratchbuf &buf = output.scratchbuf;
+
+    buf.str(opts->yystate).cstr(" = ").cstr(next);
+    append(stmts, code_stmt(alc, buf.flush()));
+
+    buf.cstr("continue");
+    if (!opts->yyloop.empty()) buf.cstr(" ").str(opts->yyloop);
+    append(stmts, code_stmt(alc, buf.flush()));
+}
+
+void emit_rule(Output &output, CodeList *stmts, const DFA &dfa, size_t rule_idx) {
+    const opt_t *opts = output.block().opts;
+    const Rule &rule = dfa.rules[rule_idx];
+    const SemAct *semact = rule.semact;
+    code_alc_t &alc = output.allocator;
+    Scratchbuf &o = output.scratchbuf;
+
+    gen_fintags(output, stmts, dfa, rule);
+
+    if (opts->target == TARGET_SKELETON) {
+        emit_skeleton_action(output, stmts, dfa, rule_idx);
+        return;
+    }
+
+    // Condition in the semantic action is the one set with => or :=> rule.
+    const std::string &cond = semact->cond.empty() ? dfa.cond : semact->cond;
+    // Next condition is either the one specified in semantic action, or the current one.
+    const char *next_cond = o.str(opts->condEnumPrefix).str(cond).flush();
+    // Next state is normally -1 (the initial storable state corresponding to no YYFILL
+    // invocation), but in the loop/switch mode conditions and storable states are both
+    // implemented via `yystate`, so the next state is the next condition.
+    const char *next_state = (!dfa.cond.empty() && opts->loop_switch) ? next_cond : "-1";
+
+    if (opts->fFlag) {
+        // Generate YYSETSTATE in the final state. This is needed because the user may
+        // enclose the lexer in an outer loop that goes via YYGETSTATE switch (it may
+        // happen if `getstate:re2c` is not used, which is the case in the loop/switch
+        // mode). The user would expect the next iteration of the loop to start in the
+        // initial DFA state, so YYGETSTATE should return the corresponding value.
+        gen_setstate(output, stmts, next_state);
+    }
+
+    if (cond != dfa.cond && !(opts->loop_switch && opts->fFlag)) {
+        // Omit YYSETCONDITION if the current condition is the same as the new one.
+        // Also omit it if both storable state and conditions are used with loop/switch:
+        // only one of YYGETSTATE and YYGETCONDITION can be used to initialize `yystate`,
+        // and it must be YYGETSTATE because the lexer may be reentered after an YYFILL
+        // invocation. Therefore we use YYSETSTATE instead of YYSETCONDITION in the final
+        // states in order to match YYGETSTATE in `yystate` initialization.
+        gen_setcondition(output, stmts, next_cond);
+    }
+
+    if (!semact->autogen) {
+        // User-defined semantic action.
+        if (!dfa.setup.empty()) append(stmts, code_text(alc, o.str(dfa.setup).flush()));
+        append(stmts, code_line_info_input(alc, semact->loc));
+        append(stmts, code_text(alc, o.str(semact->text).flush()));
+        append(stmts, code_line_info_output(alc));
+    } else if (opts->loop_switch) {
+        // Autogenerated action for the :=> rule, loop/switch mode: set `yystate` to the
+        // initial state of the next condition and continue to the head of the loop.
+        gen_continue_yyloop(output, stmts, next_cond);
+    } else {
+        // Autogenerated action for the :=> rule, goto/label mode: emit `cond:goto`
+        // configuration with `cond:goto@cond` replaced by the next condition label.
+        o.str(opts->condGoto);
+        argsubst(o.stream(), opts->condGotoParam, "cond", true, opts->condPrefix + cond);
+        append(stmts, code_text(alc, o.flush()));
+    }
+}
+
 static CodeList *gen_fill_falllback(Output &output, const DFA &dfa,
     const State *from, const State *to)
 {
     const opt_t *opts = output.block().opts;
     code_alc_t &alc = output.allocator;
-    const State *fallback;
-    tcid_t falltags;
+    Scratchbuf &buf = output.scratchbuf;
 
     DASSERT(opts->eof != NOEOF);
-    if (from->action.type == Action::INITIAL) {
-        // EOF in the initial state: EOF rule takes priority over any other rule.
-        fallback = dfa.eof_state;
-        falltags = TCID0;
-    } else if (from->rule != Rule::NONE) {
-        // EOF in accepting state: match the rule in this state.
-        fallback = dfa.finstates[from->rule];
-        falltags = from->rule_tags;
-    } else {
-        // EOF in a non-accepting state: fallback to default state.
-        fallback = dfa.defstate;
-        falltags = from->fall_tags;
-    }
+
+    tcid_t falltags;
+    const State *fallback = fallback_state_with_eof_rule(dfa, opts, from, &falltags);
 
     if (opts->stadfa) {
         DASSERT(from->go.tags == from->stadfa_tags);
@@ -335,11 +369,14 @@ static CodeList *gen_fill_falllback(Output &output, const DFA &dfa,
         gen_settags(output, fallback_trans, dfa, falltags, false /* delayed */);
 
         // go to fallback state
-        fallback->label->used = true;
-        append(fallback_trans, code_stmt(alc, output.scratchbuf.cstr("goto ")
-            .str(opts->labelPrefix).label(*fallback->label).flush()));
-    }
-    else {
+        if (!opts->loop_switch) {
+            buf.cstr("goto ").str(opts->labelPrefix).label(*fallback->label);
+            append(fallback_trans, code_stmt(alc, buf.flush()));
+        } else {
+            const char *next = buf.label(*fallback->label).flush();
+            gen_continue_yyloop(output, fallback_trans, next);
+        }
+    } else {
         // Transition can be elided, because control flow "falls through" to an
         // identical transition. Tags and skip (if present) are elided as well,
         // because the next transition covers them.
@@ -362,13 +399,18 @@ static void gen_fill(Output &output, CodeList *stmts, const DFA &dfa,
 
     // Transition to YYFILL label from the initial state dispatch.
     CodeList *goto_fill = code_list(alc);
-    const char *flabel = gen_fill_label(output, fillidx);
-    append(goto_fill, code_stmt(alc, o.cstr("goto ").cstr(flabel).flush()));
+    if (!opts->loop_switch) {
+        const char *flabel = gen_fill_label(output, fillidx);
+        append(goto_fill, code_stmt(alc, o.cstr("goto ").cstr(flabel).flush()));
+    } else {
+        const char *next = o.u32(output.block().fill_state).flush();
+        gen_continue_yyloop(output, goto_fill, next);
+    }
 
     CodeList *fill = code_list(alc);
 
     if (opts->fFlag) {
-        gen_setstate(output, eof_rule ? fill : stmts, fillidx);
+        gen_setstate(output, eof_rule ? fill : stmts, o.u32(fillidx).flush());
     }
 
     if (opts->fill_use) {
@@ -413,7 +455,7 @@ static void gen_fill(Output &output, CodeList *stmts, const DFA &dfa,
         }
     }
     if (opts->fFlag) {
-        output.block().fill_goto.push_back(goto_fill);
+        output.block().fill_goto[fillidx] = goto_fill;
     }
 
     if (opts->fill_check && fill->head) {
@@ -436,6 +478,7 @@ void gen_fill_and_label(Output &output, CodeList *stmts, const DFA &dfa, const S
 
     if (need_fill_label) {
         ++output.block().fill_index;
+        output.block().fill_state = s->label->index;
     }
 
     if (need_fill_in_state) {
@@ -451,7 +494,7 @@ void gen_fill_and_label(Output &output, CodeList *stmts, const DFA &dfa, const S
         gen_settags(output, stmts, dfa, s->go.tags, opts->stadfa /* delayed */);
     }
 
-    if (need_fill_label) {
+    if (need_fill_label && !opts->loop_switch) {
         const char *flabel = gen_fill_label(output, output.block().fill_index - 1);
         append(stmts, code_slabel(output.allocator, flabel));
     }
@@ -463,7 +506,6 @@ void gen_goto(Output &output, const DFA &dfa, CodeList *stmts, const State *from
     const opt_t *opts = output.block().opts;
     code_alc_t &alc = output.allocator;
     Scratchbuf &o = output.scratchbuf;
-    const char *text;
 
     if (jump.eof) {
         gen_fill(output, stmts, dfa, from, jump.to);
@@ -480,11 +522,14 @@ void gen_goto(Output &output, const DFA &dfa, CodeList *stmts, const State *from
     }
 
     if (!jump.elide) {
-        jump.to->label->used = true;
-        text = o.cstr("goto ").str(opts->labelPrefix).label(*jump.to->label).flush();
-        append(stmts, code_stmt(alc, text));
-    }
-    else {
+        if (!opts->loop_switch) {
+            o.cstr("goto ").str(opts->labelPrefix).label(*jump.to->label);
+            append(stmts, code_stmt(alc, o.flush()));
+        } else if (jump.to->label->used) {
+            const char *next = o.label(*jump.to->label).flush();
+            gen_continue_yyloop(output, stmts, next);
+        }
+    } else {
         // Goto can be elided, because control flow "falls through" to the
         // correct DFA state. This usually happens for the last statement in a
         // sequence of "linear if" statements.
@@ -618,8 +663,8 @@ void gen_settags(Output &output, CodeList *tag_actions, const DFA &dfa, tcid_t t
 
     for (const tcmd_t *p = cmd; p; p = p->next) {
         const tagver_t l = p->lhs, r = p->rhs, *h = p->history, *h0;
-        const std::string le = vartag_expr(l, opts);
-        const std::string re = vartag_expr(r, opts);
+        const std::string le = vartag_expr(l, opts, dfa.mtagvers);
+        const std::string re = vartag_expr(r, opts, dfa.mtagvers);
         std::string s;
 
         if (tcmd_t::iscopy(p)) {
@@ -651,7 +696,7 @@ void gen_settags(Output &output, CodeList *tag_actions, const DFA &dfa, tcid_t t
                 Scratchbuf o2(alc);
                 for (const tcmd_t *q = p; q && tcmd_t::isset(q); p = q, q = q->next) {
                     Scratchbuf &x = q->history[0] == TAGVER_BOTTOM ? o : o2;
-                    x.str(vartag_expr(q->lhs, opts)).cstr(" = ");
+                    x.str(vartag_expr(q->lhs, opts, dfa.mtagvers)).cstr(" = ");
                 }
                 if (!o.empty()) {
                     o.cstr("NULL");
@@ -683,7 +728,7 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
 
     CodeList *varops = code_list(alc);
     CodeList *fixops = code_list(alc);
-    CodeList *fixtrailops = code_list(alc);
+    CodeList *trailops = code_list(alc);
     CodeList *fixpostops = code_list(alc);
     std::string negtag;
 
@@ -697,13 +742,13 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
 
         if (!fixed(tag)) {
             // variable tag
-            const std::string expr = vartag_expr(fins[t], opts);
+            const std::string expr = vartag_expr(fins[t], opts, dfa.mtagvers);
             if (trailing(tag)) {
                 const bool notag = dfa.oldstyle_ctxmarker;
                 if (generic) {
-                    gen_restorectx(output, varops, notag ? "" : expr);
+                    gen_restorectx(output, trailops, notag ? "" : expr);
                 } else {
-                    gen_assign(output, varops, opts->yycursor,
+                    gen_assign(output, trailops, opts->yycursor,
                         notag ? opts->yyctxmarker : expr);
                 }
             } else {
@@ -716,15 +761,15 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
             const int32_t dist = static_cast<int32_t>(tag.dist);
             const bool fixed_on_cursor = tag.base == Tag::RIGHTMOST;
             const std::string base = fixed_on_cursor
-                ? opts->yycursor : vartag_expr(fins[tag.base], opts);
+                ? opts->yycursor : vartag_expr(fins[tag.base], opts, dfa.mtagvers);
 
             if (trailing(tag)) {
                 DASSERT(tag.toplevel);
                 if (generic) {
                     if (!fixed_on_cursor) {
-                        gen_restorectx(output, fixtrailops, base);
+                        gen_restorectx(output, trailops, base);
                     }
-                    gen_shift(output, fixtrailops, -dist, "", false /* unused */);
+                    gen_shift(output, trailops, -dist, "", false /* unused */);
                 } else {
                     if (!fixed_on_cursor) {
                         o.str(opts->yycursor).cstr(" = ").str(base);
@@ -732,7 +777,7 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
                     } else if (dist > 0) {
                         o.str(opts->yycursor).cstr(" -= ").i32(dist);
                     }
-                    append(fixtrailops, code_stmt(alc, o.flush()));
+                    append(trailops, code_stmt(alc, o.flush()));
                 }
             } else {
                 DASSERT(!fintags.empty());
@@ -784,9 +829,11 @@ void gen_fintags(Output &output, CodeList *stmts, const DFA &dfa, const Rule &ru
         }
     }
 
+    // Variable tags must be set before fixed tags which depend on them. Trailing context
+    // is updated after all tags, because fixed tags may depend on the rightmost position.
     append(stmts, varops);
-    append(stmts, fixtrailops);
     append(stmts, fixops);
+    append(stmts, trailops);
 
     if (!negtag.empty()) {
         // With generic API there is no explicit negative NULL value, so it is
@@ -814,17 +861,6 @@ void expand_fintags(const Tag &tag, std::vector<std::string> &fintags)
     }
 }
 
-bool endstate(const State *s)
-{
-    // 'end' state is a state which has no outgoing transitions on symbols
-    // usually 'end' states are final states (not all final states are 'end'
-    // states), but sometimes 'end' state happens to be initial non-accepting
-    // state, e.g. in case of rule '[]'
-    DASSERT(s->go.nspans > 0);
-    const Action::type_t &a = s->go.span[0].to->action.type;
-    return s->go.nspans == 1 && (a == Action::RULE || a == Action::ACCEPT);
-}
-
 const char *gen_lessthan(Scratchbuf &o, const opt_t *opts, size_t n)
 {
     if (opts->input_api == INPUT_CUSTOM) {
