@@ -54,8 +54,9 @@ static std::string make_name(Output &output, const std::string &cond, const loc_
 
 static smart_ptr<DFA> ast_to_dfa(const spec_t &spec, Output &output)
 {
-    const opt_t *opts = output.block().opts;
-    const loc_t &loc = output.block().loc;
+    OutputBlock &block = output.block();
+    const opt_t *opts = block.opts;
+    const loc_t &loc = block.loc;
     Msg &msg = output.msg;
     const std::vector<ASTRule> &rules = spec.rules;
     const std::string
@@ -71,7 +72,17 @@ static smart_ptr<DFA> ast_to_dfa(const spec_t &spec, Output &output)
     insert_default_tags(re);
     warn_nullable(re, cond);
 
-    nfa_t nfa(re);
+    size_t nfa_size, nfa_depth;
+    compute_size_and_depth(re.res, &nfa_size, &nfa_depth);
+    if (nfa_depth > MAX_NFA_DEPTH) {
+        error("NFA depth exceeds limits");
+        exit(1);
+    } else if (nfa_size > MAX_NFA_STATES) {
+        error("NFA has too many states");
+        exit(1);
+    }
+
+    nfa_t nfa(re, nfa_size);
     DDUMP_NFA(opts, nfa);
 
     dfa_t dfa(nfa, spec.def_rule, spec.eof_rule);
@@ -116,33 +127,30 @@ static smart_ptr<DFA> ast_to_dfa(const spec_t &spec, Output &output)
     adfa->prepare(opts);
     DDUMP_ADFA(opts, *adfa);
 
-    // finally gather overall DFA statistics
-    adfa->calc_stats(output.block());
-
-    // accumulate global statistics from this particular DFA
-    output.max_fill = std::max(output.max_fill, adfa->max_fill);
-    output.max_nmatch = std::max(output.max_nmatch, adfa->max_nmatch);
-    if (adfa->need_accept) {
-        output.block().used_yyaccept = true;
-    }
+    // gather overall DFA statistics and add it to the output block
+    adfa->calc_stats(block);
+    block.max_fill = std::max(block.max_fill, adfa->max_fill);
+    block.max_nmatch = std::max(block.max_nmatch, adfa->max_nmatch);
+    block.used_yyaccept = block.used_yyaccept || adfa->need_accept;
 
     return make_smart_ptr(adfa);
 }
 
 void compile(Scanner &input, Output &output, Opt &opts)
 {
-    specs_t rspecs;
-    symtab_t symtab;
+    RulesBlocks rblocks;
     const conopt_t *globopts = &opts.glob;
     code_alc_t &alc = output.allocator;
     const loc_t &loc0 = input.tok_loc();
+    std::string block_name;
+    loc_t block_loc;
 
     output.header_mode(1);
-    output.new_block(opts, loc0, false /* reuse */);
+    output.new_block(opts, INPUT_GLOBAL, block_name, loc0);
     output.wversion_time();
 
     output.header_mode(0);
-    output.new_block(opts, loc0, false /* reuse */);
+    output.new_block(opts, INPUT_GLOBAL, block_name, loc0);
     output.wversion_time();
     output.wdelay_stmt(0, code_line_info_input(alc, loc0));
 
@@ -150,36 +158,35 @@ void compile(Scanner &input, Output &output, Opt &opts)
         output.wdelay_stmt(0, emit_skeleton_prolog(output));
     }
 
-    const opt_t *rules_opts = NULL;
     const opt_t *accum_opts = output.block().opts;
 
     for (;;) {
         // parse everything up to the next re2c block
-        Scanner::ParseMode mode = input.echo(output);
-        if (mode == Scanner::Stop) break;
-        validate_mode(mode, globopts->rFlag, rules_opts, input);
+        InputBlockKind kind = input.echo(output, block_name);
+        if (kind == INPUT_ERROR) exit(1);
+        if (kind == INPUT_END) break;
 
         // parse the next re2c block
         specs_t specs;
-        if (mode == Scanner::Reuse) {
-            specs = rspecs;
-            opts.restore(rules_opts);
+        if (kind == INPUT_USE) {
+            const RulesBlock *rb = rblocks.find(block_name);
+            if (rb == NULL) exit(1);
+            specs = rb->specs;
+            opts.restore(rb->opts);
             output.state_goto = false;
         }
         output.cond_goto = false;
-        parse(input, specs, symtab, opts);
+        block_loc = input.tok_loc();
+        parse(input, specs, opts, rblocks);
 
         // start new output block with accumulated options
-        const loc_t &loc = input.cur_loc();
-        output.new_block(opts, loc, mode == Scanner::Reuse);
+        output.new_block(opts, kind, block_name, block_loc);
 
-        if (mode == Scanner::Rules) {
+        if (kind == INPUT_RULES) {
             // save AST and options for future use
-            rspecs = specs;
-            rules_opts = output.block().opts;
+            rblocks.add(block_name, output.block().opts, specs);
         } else {
-            validate_ast(specs, output.block().opts, output.msg);
-            normalize_ast(specs);
+            check_and_merge_special_rules(specs, output.block().opts, output.msg);
 
             // compile AST to DFA
             dfas_t dfas;
@@ -190,20 +197,30 @@ void compile(Scanner &input, Output &output, Opt &opts)
             // compile DFA to code
             gen_code(output, dfas);
         }
-        output.wdelay_stmt(0, code_line_info_input(alc, loc));
+        output.wdelay_stmt(0, code_line_info_input(alc, input.cur_loc()));
 
         // accumulate whole-program information from this block
         output.gather_info_from_block();
 
-        // accumulated whole-program options exclude rules/reuse blocks
-        if (mode == Scanner::Parse) {
+        // Do not accumulate whole-program options for rules/reuse/local blocks.
+        // Global blocks add their named definitions and configurations to the
+        // global scope, local blocks don't. Historically global is the default.
+        if (kind == INPUT_GLOBAL) {
             accum_opts = output.block().opts;
         } else {
             opts.restore(accum_opts);
         }
     }
 
-    output.total_opts = accum_opts ? accum_opts : rules_opts;
+    output.total_opts = accum_opts ? accum_opts : rblocks.last_opts();
+
+    // For special targets (skeleton and .dot) merge header into the output file.
+    if (globopts->target != TARGET_CODE && output.need_header) {
+        output.need_header = false;
+        blocks_t &cblocks = output.cblocks, &hblocks = output.hblocks;
+        cblocks.insert(cblocks.end(), hblocks.begin(), hblocks.end());
+        hblocks.clear();
+    }
 
     if (globopts->target == TARGET_SKELETON) {
         output.wdelay_stmt(0, emit_skeleton_epilog(output));
