@@ -14,8 +14,8 @@ using namespace re2c;
 int main(int, char *argv[])
 {
     conopt_t globopts;
-    Opt opts(globopts);
     Msg msg;
+    Opt opts(globopts, msg);
 
     switch (parse_opts(argv, globopts, opts, msg)) {
         case OK:        break;
@@ -40,6 +40,10 @@ int main(int, char *argv[])
         return 1;
     }
 
+    if (!scanner.gen_dep_file()) {
+        return 1;
+    }
+
     if (globopts.verbose) {
         fprintf(stderr, "re2c: success\n");
     }
