@@ -24,7 +24,7 @@ public:
     inline Msg(): filenames(), warn(*this), locfmt(LOCFMT_GNU) {}
 
     size_t register_filename(const std::string &filename);
-    void fatal(const loc_t &loc, const char *fmt, ...) RE2C_ATTR((format (printf, 3, 4), noreturn));
+    void error(const loc_t &loc, const char *fmt, ...) RE2C_ATTR((format (printf, 3, 4)));
     void warning(const char *type, const loc_t &loc, bool error, const char *fmt, ...) RE2C_ATTR((format (printf, 5, 6)));
 
     friend class Warn;
@@ -36,7 +36,6 @@ private:
 };
 
 void error(const char *fmt, ...) RE2C_ATTR((format (printf, 1, 2)));
-void fatal(const char *fmt, ...) RE2C_ATTR((format (printf, 1, 2), noreturn));
 void error_arg(const char *option);
 void usage ();
 void vernum ();
