@@ -1,3 +1,5 @@
+#include <stdlib.h>
+
 #include "src/msg/msg.h"
 #include "src/options/opt.h"
 
@@ -7,12 +9,17 @@ namespace re2c {
 // This function should only change global options.
 static void fix_conopt(conopt_t &glob)
 {
-    if (glob.target == TARGET_SKELETON) {
+    if (glob.target == TARGET_DOT) {
+        glob.iFlag = true;
+    } else if (glob.target == TARGET_SKELETON) {
         glob.fFlag = false;
+        glob.iFlag = true;
     }
 
-    if (!glob.lookahead) {
-        glob.eager_skip = true;
+    if (glob.lang == LANG_RUST) {
+        glob.loop_switch = true;
+        // No line directives in Rust: https://github.com/rust-lang/rfcs/issues/1862
+        glob.iFlag = true;
     }
 
     // append directory separator '/' to all paths that do not have it
@@ -24,6 +31,13 @@ static void fix_conopt(conopt_t &glob)
         }
     }
 
+    if (!glob.lookahead || glob.loop_switch) {
+        // for loop-switch enable eager-skip always (not only in cases when
+        // YYFILL labels are used) to avoid special handling of initial state
+        // when there are transitions into it.
+        glob.eager_skip = true;
+    }
+
     if (!glob.dep_file.empty() && glob.output_file.empty()) {
         error("cannot generate dep file, output file not specified");
         exit(1);
@@ -34,10 +48,11 @@ static void fix_conopt(conopt_t &glob)
 // options).
 static void fix_mutopt_defaults(const conopt_t &glob, mutopt_t &defaults)
 {
-    // For the Go backend, generic API is set by default, because "default" C
-    // API with pointers doesn't work (there is no pointer arithmetics in Go).
-    // To make the default Go API less restrictive, decorations are disabled.
-    if (glob.lang == LANG_GO) {
+    // For the Go and Rust backends generic API is set by default, because the default C
+    // API with pointers doesn't work (there is no pointer arithmetics in Go, and in Rust
+    // it is different enough from C). Use freeform generic API by default to make it less
+    // restrictive.
+    if (glob.lang != LANG_C) {
         defaults.input_api = INPUT_CUSTOM;
         defaults.api_style = API_FREEFORM;
     }
@@ -56,8 +71,6 @@ static void fix_mutopt(const conopt_t &glob, const mutopt_t &defaults,
     if (glob.target != TARGET_CODE) {
         // output files
         real.header_file = "";
-        // default line information
-        real.iFlag = defaults.iFlag;
         // default environment-sensitive formatting
         real.topIndent = defaults.topIndent;
         real.indString = defaults.indString;
@@ -117,6 +130,8 @@ static void fix_mutopt(const conopt_t &glob, const mutopt_t &defaults,
         real.yych = defaults.yych;
         real.bEmitYYCh = defaults.bEmitYYCh;
         real.yychConversion = defaults.yychConversion;
+        real.yyloop = defaults.yyloop;
+        real.yystate = defaults.yystate;
         real.fill = defaults.fill;
         real.fill_use = defaults.fill_use;
         real.fill_check = defaults.fill_check;
@@ -153,9 +168,11 @@ static void fix_mutopt(const conopt_t &glob, const mutopt_t &defaults,
         real.yyfilllabel = defaults.yyfilllabel;
         real.yynext = defaults.yynext;
         real.yyaccept = defaults.yyaccept;
-        real.bUseStateAbort = defaults.bUseStateAbort;
         real.bUseStateNext = defaults.bUseStateNext;
     }
+    if (!glob.fFlag && !glob.loop_switch) {
+        real.bUseStateAbort = defaults.bUseStateAbort;
+    }
     if (real.posix_semantics) {
         real.posix_syntax = true;
     }
@@ -206,11 +223,7 @@ static void fix_mutopt(const conopt_t &glob, const mutopt_t &defaults,
     }
 
     // set implied options
-    if (glob.target == TARGET_DOT) {
-        real.iFlag = true;
-    }
-    else if (glob.target == TARGET_SKELETON) {
-        real.iFlag = true;
+    if (glob.target == TARGET_SKELETON) {
         real.input_api = INPUT_CUSTOM;
         real.indString = "    ";
         real.topIndent = 2;
@@ -249,26 +262,40 @@ static void fix_mutopt(const conopt_t &glob, const mutopt_t &defaults,
     if (real.fill_naked) {
         real.fill_arg_use = false;
     }
+    if (glob.loop_switch) {
+        // With --loop-switch there is no `goto`.
+        real.yyctable = defaults.yyctable;
+        real.condDivider = defaults.condDivider;
+        real.condDividerParam = defaults.condDividerParam;
+        real.condGoto = defaults.condGoto;
+        real.condGotoParam = defaults.condGotoParam;
+    }
+    if (glob.lang == LANG_RUST) {
+        // In Rust constants should be uppercase.
+        if (is_default.condEnumPrefix) real.condEnumPrefix = "YYC_";
+        // In Rust `continue` statements have labels, use it to avoid ambiguity.
+        if (is_default.yyloop) real.yyloop = "'yyl";
+    } else if (glob.lang == LANG_GO) {
+        // In Go `continue` statements have labels, use it to avoid ambiguity.
+        if (is_default.yyloop) real.yyloop = "yyl";
+    }
 
     // errors
-    if (glob.target == TARGET_SKELETON && glob.lang != LANG_C) {
-        error("skeleton is not supported for non-C language backends");
-        exit(1);
-    }
-    if (glob.lang == LANG_GO) {
+    if (glob.lang != LANG_C) {
+        if (glob.target == TARGET_SKELETON) {
+            error("skeleton is not supported for non-C backends");
+            exit(1);
+        }
         if (real.input_api == INPUT_DEFAULT) {
-            error("default C API is not supported for the Go backend,"
-                " as Go has no pointer arithmetics");
+            error("pointer API is not supported for non-C backends");
             exit(1);
         }
         if (real.gFlag) {
-            error("-g, --computed-gotos option is not supported for the Go backend,"
-                " as Go does not have computed goto");
+            error("-g, --computed-gotos option is not supported for non-C backends");
             exit(1);
         }
         if (real.case_ranges) {
-            error("--case-ranges option is not supported for the Go backend,"
-                " as Go has no case ranges");
+            error("--case-ranges option is not supported for non-C backends");
             exit(1);
         }
     }
@@ -311,10 +338,31 @@ static void fix_mutopt(const conopt_t &glob, const mutopt_t &defaults,
         error("storable state requires YYFILL to be enabled");
         exit(1);
     }
+    if (glob.loop_switch) {
+        if (real.gFlag) {
+            error("cannot combine loop switch and computed gotos");
+            exit(1);
+        }
+        if (real.bFlag) {
+            // TODO: generate bitmaps before the joined loop/switch for all conditions.
+            error("bitmaps with loop switch are not supported");
+            exit(1);
+        }
+        if (!glob.lookahead) {
+            // The problem here is that TDFA(0) may have initial register operations that
+            // should be executed on DFA entry. With loop/switch these operations would
+            // need to go in a special initial state (they can't go before the `yystate`
+            // loop because that would not work with start conditions).
+            error("TDFA(0) with loop switch is not supported");
+            exit(1);
+        }
+    }
 }
 
-Opt::Opt(const conopt_t &globopts)
+Opt::Opt(const conopt_t &globopts, Msg &msg)
     : glob(globopts)
+    , symtab()
+    , msg(msg)
     , defaults()
     , is_default()
     , user()
@@ -365,7 +413,7 @@ void Opt::sync()
 const opt_t *Opt::snapshot()
 {
     sync();
-    return new opt_t(glob, real, is_default);
+    return new opt_t(glob, real, is_default, symtab);
 }
 
 void Opt::restore(const opt_t *opts)
@@ -378,6 +426,26 @@ void Opt::restore(const opt_t *opts)
 #undef MUTOPT1
 #undef MUTOPT
 
+    symtab = opts->symtab;
+
+    diverge = true;
+    sync();
+}
+
+void Opt::merge(const opt_t *opts, const loc_t &loc)
+{
+#define MUTOPT1 MUTOPT
+#define MUTOPT(type, name, value) \
+    if (!opts->is_default_##name) { \
+        user.name = opts->name; \
+        is_default.name = false; \
+    }
+    RE2C_MUTOPTS
+#undef MUTOPT1
+#undef MUTOPT
+
+    merge_symtab(symtab, opts->symtab, loc, msg);
+
     diverge = true;
     sync();
 }
@@ -400,20 +468,16 @@ RE2C_MUTOPTS
 #undef MUTOPT1
 #undef MUTOPT
 
-void Opt::set_encoding(Enc::type_t t)
-{
-    user.encoding.set(t);
+void Opt::set_encoding(Enc::type_t type, bool on) {
+    if (on) {
+        user.encoding.set(type);
+    } else {
+        user.encoding.unset(type);
+    }
     is_default.encoding = false;
     diverge = true;
 }
 
-void Opt::unset_encoding(Enc::type_t t)
-{
-    user.encoding.unset(t);
-    is_default.encoding = false; // explicitly unset by the user => not default
-    diverge = true;
-}
-
 void Opt::set_encoding_policy(Enc::policy_t p)
 {
     user.encoding.setPolicy(p);
