@@ -6,9 +6,9 @@
 #include <string>
 #include <vector>
 
-#include "src/dfa/dfa.h"
-#include "src/regexp/empty_class_policy.h"
+#include "src/constants.h"
 #include "src/encoding/enc.h"
+#include "src/options/symtab.h"
 #include "src/util/forbid_copy.h"
 
 
@@ -38,41 +38,6 @@ namespace re2c {
 
 class Msg;
 
-enum target_t {
-    TARGET_CODE,
-    TARGET_DOT,
-    TARGET_SKELETON
-};
-
-enum lang_t {
-    LANG_C,
-    LANG_GO
-};
-
-enum input_api_t {
-    INPUT_DEFAULT,
-    INPUT_CUSTOM
-};
-
-enum api_style_t {
-    API_FUNCTIONS,
-    API_FREEFORM
-};
-
-enum fixed_tags_t {
-    FIXTAG_NONE,
-    FIXTAG_TOPLEVEL,
-    FIXTAG_ALL
-};
-
-enum parse_opts_t {
-    OK,
-    EXIT_OK,
-    EXIT_FAIL
-};
-
-const uint32_t NOEOF = ~0u - 1;
-
 #define RE2C_SIGIL "@@"
 
 // default backend is C
@@ -87,7 +52,6 @@ const uint32_t NOEOF = ~0u - 1;
     CONSTOPT (bool, version, true) \
     CONSTOPT (bool, cFlag, false) \
     CONSTOPT (bool, fFlag, false) \
-    CONSTOPT (bool, rFlag, false) \
     CONSTOPT (bool, FFlag, false) \
     CONSTOPT (bool, verbose, false) \
     /* files */ \
@@ -98,17 +62,20 @@ const uint32_t NOEOF = ~0u - 1;
     CONSTOPT (std::vector<std::string>, incpaths, std::vector<std::string>()) \
     /* input encoding */ \
     CONSTOPT (Enc::type_t, input_encoding, Enc::ASCII) \
+    /* #line directives */ \
+    CONSTOPT (bool, iFlag, false) \
     /* internals */ \
     CONSTOPT (dfa_minimization_t, dfa_minimization, DFA_MINIMIZATION_MOORE) \
     CONSTOPT (posix_closure_t, posix_closure, POSIX_CLOSURE_GOR1) \
     CONSTOPT (posix_prectable_alg_t, posix_prectable, POSIX_PRECTABLE_COMPLEX) \
     CONSTOPT (fixed_tags_t, fixed_tags, FIXTAG_ALL) \
     CONSTOPT (bool, lookahead, true) \
-    CONSTOPT (bool, eager_skip, false) \
     CONSTOPT (bool, optimize_tags, true) \
     CONSTOPT (bool, backward, false) \
     CONSTOPT (bool, stadfa, false) \
     CONSTOPT (bool, nested_negative_tags, true) \
+    CONSTOPT (bool, eager_skip, false) \
+    CONSTOPT (bool, loop_switch, false) \
     /* debug */ \
     CONSTOPT (bool, dump_nfa, false) \
     CONSTOPT (bool, dump_dfa_raw, false) \
@@ -161,6 +128,8 @@ const uint32_t NOEOF = ~0u - 1;
     MUTOPT (std::string, tags_expression, RE2C_SIGIL) \
     MUTOPT (bool, posix_syntax, false) \
     MUTOPT (bool, posix_semantics, false) \
+    MUTOPT (bool, subhistories, false) \
+    MUTOPT (bool, autotags, false) \
     /* code generation */ \
     MUTOPT (bool, sFlag, false) \
     MUTOPT (bool, bFlag, false) \
@@ -168,6 +137,9 @@ const uint32_t NOEOF = ~0u - 1;
     MUTOPT (bool, yybmHexTable, false) \
     MUTOPT (bool, gFlag, false) \
     MUTOPT (bool, case_ranges, false) \
+    MUTOPT (bool, unsafe, true) \
+    MUTOPT (std::string, yyloop, "") \
+    MUTOPT (std::string, yystate, "yystate") \
     MUTOPT (std::string, yytarget, "yytarget") \
     MUTOPT (uint32_t, cGotoThreshold, 9) \
     MUTOPT (uint32_t, eof, NOEOF) \
@@ -198,8 +170,6 @@ const uint32_t NOEOF = ~0u - 1;
     MUTOPT (std::string, yyshiftmtag, "YYSHIFTMTAG") \
     MUTOPT (api_style_t, api_style, API_FUNCTIONS) \
     MUTOPT (std::string, api_sigil, RE2C_SIGIL) \
-    /* #line directives */ \
-    MUTOPT (bool, iFlag, false) \
     /* debug */ \
     MUTOPT (bool, dFlag, false) \
     MUTOPT (std::string, yydebug, "YYDEBUG") \
@@ -291,7 +261,10 @@ struct opt_t {
 #undef MUTOPT1
 #undef MUTOPT
 
-    opt_t(const conopt_t &con, const mutopt_t &mut, const mutdef_t &def)
+    const symtab_t symtab;
+
+    opt_t(const conopt_t &con, const mutopt_t &mut, const mutdef_t &def,
+        const symtab_t symtab)
 #define CONSTOPT1(type, name, value) : name(con.name)
 #define CONSTOPT(type, name, value)  , name(con.name)
         RE2C_CONSTOPTS
@@ -306,13 +279,52 @@ struct opt_t {
         RE2C_MUTOPTS
 #undef MUTOPT1
 #undef MUTOPT
+        , symtab(symtab)
     {}
+
+    opt_t(const opt_t &opt)
+#define CONSTOPT1(type, name, value) : name(opt.name)
+#define CONSTOPT(type, name, value)  , name(opt.name)
+        RE2C_CONSTOPTS
+#undef CONSTOPT1
+#undef CONSTOPT
+#define MUTOPT1 MUTOPT
+#define MUTOPT(type, name, value) , name(opt.name)
+        RE2C_MUTOPTS
+#undef MUTOPT
+#define MUTOPT(type, name, value) , is_default_##name(opt.is_default_##name)
+        RE2C_MUTOPTS
+#undef MUTOPT1
+#undef MUTOPT
+        , symtab(opt.symtab)
+    {}
+
+    opt_t& operator=(const opt_t &opt)
+    {
+#define CONSTOPT1 CONSTOPT
+#define CONSTOPT(type, name, value) name = opt.name;
+        RE2C_CONSTOPTS
+#undef CONSTOPT1
+#undef CONSTOPT
+#define MUTOPT1 MUTOPT
+#define MUTOPT(type, name, value) name = opt.name;
+        RE2C_MUTOPTS
+#undef MUTOPT
+#define MUTOPT(type, name, value) is_default_##name = opt.is_default_##name;
+        RE2C_MUTOPTS
+#undef MUTOPT1
+#undef MUTOPT
+        const_cast<symtab_t&>(symtab) = opt.symtab;
+        return *this;
+    }
 };
 
 // Options management.
 struct Opt {
 public:
     const conopt_t &glob;
+    symtab_t symtab;
+    Msg &msg;
 
 private:
     // Default mutable options. They depend on the global options, such as the
@@ -339,10 +351,11 @@ private:
     bool diverge;
 
 public:
-    explicit Opt(const conopt_t &globopts);
+    Opt(const conopt_t &globopts, Msg &msg);
     const opt_t *snapshot();
     void fix_global_and_defaults();
     void restore(const opt_t *opts);
+    void merge(const opt_t *opts, const loc_t &loc);
 
 #define MUTOPT1 MUTOPT
 #define MUTOPT(type, name, value) \
@@ -351,8 +364,7 @@ public:
     RE2C_MUTOPTS
 #undef MUTOPT1
 #undef MUTOPT
-    void set_encoding(Enc::type_t t);
-    void unset_encoding(Enc::type_t t);
+    void set_encoding(Enc::type_t type, bool on);
     void set_encoding_policy(Enc::policy_t p);
 
     void reset_group_startlabel();
