@@ -7,27 +7,35 @@
 #include <string>
 #include <vector>
 
+#include "src/constants.h"
 #include "src/debug/debug.h"
 #include "src/msg/location.h"
 #include "src/parse/input.h"
 #include "src/parse/lex.h"
-#include "src/options/opt.h"
 #include "src/encoding/enc.h"
 #include "src/encoding/utf8/utf8.h"
 #include "src/util/forbid_copy.h"
+#include "src/util/slab_allocator.h"
 
 
 namespace re2c {
 
 class Msg;
-class Output;
+struct Output;
 struct AST;
 struct ASTChar;
+struct BlockNameList;
+struct conopt_t;
+struct opt_t;
+struct Opt;
+
+// Must be defined exacly as in codegen.
+typedef slab_allocator_t<1024 * 1024, 8> code_alc_t;
 
 class Scanner: private ScannerState
 {
 public:
-    enum ParseMode {Stop, Parse, Reuse, Rules};
+    enum ParseMode {Stop, Global, Local, Reuse, Rules, Error};
     static const char *const ENDPOS;
 
     Msg &msg;
@@ -42,11 +50,11 @@ public:
     Scanner(const conopt_t *o, Msg &m);
     ~Scanner();
     bool open(const std::string &filename, const std::string *parent);
-    bool include(const std::string &filename);
+    bool include(const std::string &filename, char *at);
     bool gen_dep_file() const;
     const loc_t &tok_loc() const;
     loc_t cur_loc() const;
-    ParseMode echo(Output &out);
+    InputBlockKind echo(Output &out, std::string &block_name);
     int scan();
     void lex_conf(Opt &opts);
 
@@ -61,16 +69,19 @@ private:
     inline void set_line(uint32_t l);
     inline void next_line();
     void set_sourceline ();
-    void lex_end_of_comment(Output &out);
+    bool lex_opt_name(std::string &name);
+    bool lex_name_list(code_alc_t &alc, BlockNameList **ptail);
+    bool lex_block(Output &out, CodeKind kind, uint32_t indent, uint32_t mask);
+    bool lex_block_end(Output &out, bool allow_garbage = false);
     void lex_code_indented();
     void lex_code_in_braces();
+    void try_lex_string_in_code(char quote);
     void lex_c_comment();
     void lex_cpp_comment();
     bool lex_namedef_context_re2c();
     bool lex_namedef_context_flex();
     int lex_clist();
     void lex_string(char delim);
-    void lex_tags(Output &out, bool mtags);
     uint32_t lex_cls_chr();
     bool lex_str_chr(char quote, ASTChar &ast);
     const AST *lex_cls(bool neg);
@@ -78,7 +89,6 @@ private:
     void lex_conf_encoding_policy(Opt &opts);
     void lex_conf_input(Opt &opts);
     void lex_conf_empty_class(Opt &opts);
-    void lex_conf_enc(Enc::type_t enc, Opt &opts);
     void lex_conf_api_style(Opt &opts);
     void lex_conf_assign();
     void lex_conf_semicolon();
@@ -145,11 +155,13 @@ inline const Input& Scanner::get_cinput() const
     return *files[get_input_index()];
 }
 
-inline uint32_t Scanner::decode(const char *str) const
-{
-    return globopts->input_encoding == Enc::ASCII
-        ? static_cast<uint8_t>(str[0])
-        : utf8::decode_unsafe(str);
+inline bool Scanner::lex_conf_bool() {
+    return lex_conf_number() != 0;
+}
+
+inline uint32_t Scanner::lex_conf_eof() {
+    const int32_t n = lex_conf_number();
+    return n < 0 ? NOEOF : static_cast<uint32_t>(n);
 }
 
 } // end namespace re2c
