@@ -10,6 +10,7 @@
 #include "src/debug/debug.h"
 #include "src/msg/msg.h"
 #include "src/msg/warn.h"
+#include "src/options/opt.h"
 #include "src/regexp/rule.h"
 #include "src/skeleton/path.h"
 #include "src/skeleton/skeleton.h"
@@ -29,23 +30,20 @@ struct StackItem {
     Node::arcs_t::const_iterator arc;
 };
 
-static void fprint_default_arc(FILE *f, const Node::arc_t &arc)
+static void fprint_default_arc(FILE *f, const Node::range_t *r)
 {
-    const size_t ranges = arc.size();
-    if (ranges == 1 && arc[0].lower == arc[0].upper) {
-        fprintf(f, "\\x%X", arc[0].lower);
-    }
-    else {
+    if (r->next == r && r->lower == r->upper) {
+        fprintf(f, "\\x%X", r->lower);
+    } else {
         fprintf(f, "[");
-        for (size_t i = 0; i < ranges; ++i) {
-            const uint32_t
-                l = arc[i].lower,
-                u = arc[i].upper;
-            fprintf(f, "\\x%X", l);
-            if (l != u) {
-                fprintf(f, "-\\x%X", u);
+        const Node::range_t *r0 = r;
+        do {
+            fprintf(f, "\\x%X", r->lower);
+            if (r->lower != r->upper) {
+                fprintf(f, "-\\x%X", r->upper);
             }
-        }
+            r = r->next;
+        } while (r != r0);
         fprintf(f, "]");
     }
 }
@@ -68,6 +66,7 @@ void warn_undefined_control_flow(const Skeleton &skel)
     std::valarray<bool> loops(skel.nodes_count);
     std::vector<path_t> paths;
     paths_size_t size(paths_size_t::from32(0u));
+    bool use_eof_rule = skel.opts->eof != NOEOF;
 
     std::vector<StackItem> stack;
     path_t path(0);
@@ -82,10 +81,10 @@ void warn_undefined_control_flow(const Skeleton &skel)
 
         if (i.arc == node.arcs.begin()) {
             // DFS recursive enter
-            if (node.rule != Rule::NONE) {
+            if (node.rule != Rule::NONE && node.rule != skel.eof_rule) {
                 // accepting path, terminate recursion
             }
-            else if (node.end()) {
+            else if (node.end() || (use_eof_rule && node.rule == Rule::NONE)) {
                 // found path to default state
                 get_path_on_stack(path, stack, i.node);
                 paths.push_back(path);
@@ -140,8 +139,7 @@ void fprint_default_path(FILE *f, const Skeleton &skel, const path_t &p)
         if (i > 0) {
             fprintf(f, " ");
         }
-        const Node::arc_t &arc = p.arc(skel, i);
-        fprint_default_arc(stderr, arc);
+        fprint_default_arc(stderr, p.arc(skel, i));
     }
     fprintf(f, "'");
 }
