@@ -13,6 +13,8 @@
 #include "src/regexp/rule.h"
 #include "src/regexp/tag.h"
 #include "src/skeleton/skeleton.h"
+#include "src/util/file_utils.h"
+#include "src/util/string_utils.h"
 
 
 namespace re2c {
@@ -150,7 +152,7 @@ static void emit_skeleton_defines(Output &output, CodeList *code, const DFA &dfa
     append(code, code_textraw(alc, "#define YYSHIFT(o) cursor += o"));
     append(code, code_textraw(alc, "#define YYLESSTHAN(n) (limit - cursor) < n"));
     append(code, code_textraw(alc, opts->eof == NOEOF
-        ? "#define YYFILL(n) { break; }"
+        ? "#define YYFILL(n) { goto loop_end; }"
         : "#define YYFILL(n) 1 /* fail */"));
     append(code, code_newline(alc));
 }
@@ -278,22 +280,22 @@ static void emit_skeleton_stags(Output &output, CodeList *code, const DFA &dfa)
     append(code, code_block(alc, body, CodeBlock::WRAPPED));
 }
 
-static void emit_skeleton_mtags(Output &output, CodeList *code, const DFA &dfa)
+static void emit_skeleton_mtag_defs(Output &output, CodeList *code)
 {
-    if (dfa.mtagvars.empty()) return;
-
     code_alc_t &alc = output.allocator;
-    Scratchbuf &o = output.scratchbuf;
     CodeArgs *args;
-    CodeList *if_code, *block, *body;
-    const char *if_cond, *text;
+    CodeList *if_code, *block;
+    const char *if_cond;
+
+    // Use `void *` instead of `YYCTYPE *`, as YYCTYPE may vary for different
+    // blocks. and this definition must be valid throughout the whole file.
 
     append(code, code_newline(alc));
 
     append(code, code_text(alc, "typedef struct yymtag_t {"));
     block = code_list(alc);
     append(block, code_text(alc, "ptrdiff_t pred;"));
-    append(block, code_text(alc, "const YYCTYPE *elem;"));
+    append(block, code_text(alc, "const void *elem;"));
     append(code, code_block(alc, block, CodeBlock::INDENTED));
     append(code, code_text(alc, "} yymtag_t;"));
 
@@ -363,7 +365,7 @@ static void emit_skeleton_mtags(Output &output, CodeList *code, const DFA &dfa)
 
     args = code_args(alc);
     append(args, code_arg(alc, "ptrdiff_t *pt"));
-    append(args, code_arg(alc, "const YYCTYPE *t"));
+    append(args, code_arg(alc, "const void *t"));
     append(args, code_arg(alc, "yymtagpool_t *tp"));
     append(code, code_fdecl(alc, "static void yymtag", args));
     block = code_list(alc);
@@ -374,6 +376,27 @@ static void emit_skeleton_mtags(Output &output, CodeList *code, const DFA &dfa)
     append(code, code_block(alc, block, CodeBlock::WRAPPED));
 
     append(code, code_newline(alc));
+}
+
+static void emit_skeleton_mtags(Output &output, CodeList *code, const DFA &dfa)
+{
+    if (dfa.mtagvars.empty()) return;
+
+    code_alc_t &alc = output.allocator;
+    Scratchbuf &o = output.scratchbuf;
+    CodeArgs *args;
+    CodeList *if_code, *block, *body;
+    const char *if_cond, *text;
+
+    // Data structures for m-tags are the same for all blocks in the file. They
+    // are emitted at the first block that has m-tags, because it's impossible
+    // to know earlier (when generating prologue) if m-tags are needed at all.
+    if (!output.done_mtag_defs) {
+        output.done_mtag_defs = true;
+        emit_skeleton_mtag_defs(output, code);
+    }
+
+    append(code, code_newline(alc));
 
     args = code_args(alc);
     append(args, code_arg(alc, "unsigned *pkix"));
@@ -498,6 +521,7 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     if (filename.empty()) {
         filename = "<stdout>";
     }
+    filename = escape_backslashes(filename);
 
     append(code, code_newline(alc));
 
@@ -586,8 +610,17 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
 
     append(block, code_textraw(alc, ""));
 
-    append(block, code_text(alc,
-        "for (i = 0; status == 0 && cursor < eof && i < keys_count;) {"));
+    // Don't use `for` loops or `while` loops: the generated code may contain an
+    // inner loop if the --loop-switch option is used, and then `continue` and
+    // `break` statements may result in unexpected control flow. Use `goto` loop.
+    append(block, code_stmt(alc, "i = 0"));
+    append(block, code_slabel(alc, "loop"));
+
+    block2 = code_list(alc);
+    append(block2, code_stmt(alc, "goto loop_end"));
+    append(block, code_if_then_else(alc,
+        "!(status == 0 && cursor < eof && i < keys_count)", block2, NULL));
+
     block2 = code_list(alc);
     append(block2, code_stmt(alc, "token = cursor"));
     if (dfa.need_backup) {
@@ -600,11 +633,14 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     if (dfa.need_accept) {
         append(block2, code_stmt(alc, "unsigned int yyaccept = 0"));
     }
+    if (opts->loop_switch) {
+        append(block2, code_stmt(alc, o.cstr("unsigned int yystate = 0").flush()));
+    }
     // autogenerated stag variables
     if (!dfa.stagnames.empty()) {
         text = o.cstr("\n").str(indent(2, opts->indString))
             .cstr("const YYCTYPE *@@ = NULL;").flush();
-        Code *stags = code_tags(alc, text, "", false);
+        Code *stags = code_fmt(alc, CODE_STAGS, NULL, text, NULL);
         gen_tags(o, opts, stags, dfa.stagnames);
         append(block2, stags);
         append(block2, code_textraw(alc, ""));
@@ -628,7 +664,7 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
 
         text = o.cstr("\n").str(indent(2, opts->indString))
             .cstr("ptrdiff_t @@ = -1;").flush();
-        Code *mtags = code_tags(alc, text, "", true);
+        Code *mtags = code_fmt(alc, CODE_MTAGS, NULL, text, NULL);
         gen_tags(o, opts, mtags, dfa.mtagnames);
         append(block2, mtags);
         append(block2, code_textraw(alc, ""));
@@ -652,11 +688,20 @@ static void emit_skeleton_function_lex(Output &output, CodeList *code, DFA &dfa)
     }
     append(block2, code_textraw(alc, ""));
 
-    dfa.emit_body(output, block2);
+    if (!opts->loop_switch) {
+        gen_dfa_as_blocks_with_labels(output, dfa, block2);
+    } else {
+        // With loop/switch there are no labels, and each block has its own state switch.
+        // Restart state counter from zero so that cases start from zero.
+        output.label_counter = 0;
+        CodeCases *cases = code_cases(alc);
+        gen_dfa_as_switch_cases(output, dfa, cases);
+        wrap_dfas_in_loop_switch(output, block2, cases);
+    }
     append(block2, code_textraw(alc, ""));
 
-    append(block, code_block(alc, block2, CodeBlock::INDENTED));
-    append(block, code_text(alc, "}"));
+    append(block, code_block(alc, block2, CodeBlock::WRAPPED));
+    append(block, code_slabel(alc, "loop_end"));
 
     if_code = code_list(alc);
 
@@ -813,7 +858,7 @@ void emit_skeleton_action(Output &output, CodeList *code, const DFA &dfa, size_t
 
     append(code, code_block(alc, hangafter, CodeBlock::INDENTED));
 
-    append(code, code_stmt(alc, "continue"));
+    append(code, code_stmt(alc, "goto loop"));
 }
 
 } // namespace re2c
